#!/usr/bin/env python

import logging
import logging.config
import sys


class _ExcludeErrorsFilter(logging.Filter):
    def filter(self, record):
        """Filters out log messages with log level ERROR (numeric value: 40) or higher."""
        return record.levelno < 40


class Logging(object):
    config = {
        'version': 1,
        'filters': {
            'exclude_errors': {
                '()': _ExcludeErrorsFilter
            }
        },
        'formatters': {
            'include_process': {
                'format': '(%(process)d) %(asctime)s %(name)s (line %(lineno)s) | %(asctime)s %(levelname)-8s %(message)s'
            }
        },
        'handlers': {
            'console_stderr': {
                # Sends log messages with log level ERROR or higher to stderr
                'class': 'logging.StreamHandler',
                'level': 'ERROR',
                'formatter': 'include_process',
                'stream': sys.stderr
            },
            'console_stdout': {
                # Sends log messages with log level lower than ERROR to stdout
                'class': 'logging.StreamHandler',
                'level': 'DEBUG',
                'formatter': 'include_process',
                'filters': ['exclude_errors'],
                'stream': sys.stdout
            },
            'file': {
                # Sends all log messages to a file
                'class': 'logging.FileHandler',
                'level': 'DEBUG',
                'formatter': 'include_process',
                'filename': 'uploader.log',
                'encoding': 'utf8'
            }
        },
        'root': {
            # In general, this should be kept at 'NOTSET'.
            # Otherwise it would interfere with the log levels set for each handler.
            'level': 'NOTSET',
            'handlers': ['console_stderr', 'console_stdout', 'file']
        },
    }

    def get_log_file(self):
        return self.config["handlers"]["file"]["filename"]

    def set_log_file(self, log_file_path):
        self.config["handlers"]["file"]["filename"] = log_file_path

    def initialize(self):
        logging.config.dictConfig(self.config)


class UploaderException(BaseException):
    pass



#!/usr/bin/env python

import glob
import logging
import os
import re
import time
from abc import ABCMeta, abstractmethod

from soyuz.data.files import WesDataFile, RawDataFile, QcDataFile, PersonalisDataFile, PersonalisQcDataFile
from soyuz.utils import UploaderException


class WatchDirectory(object):
    UPLOADED_FILE = ".uploaded"

    def __init__(self, path):
        self.__path = path

    def is_uploaded(self, folder):
        return str(folder) in self.__read_file(self.__uploaded_file)

    @property
    def __uploaded_file(self):
        return os.path.join(self.__path, self.UPLOADED_FILE)

    @property
    def __copied_file(self):
        return os.path.join(self.__path, self.COPIED_FILE)

    def get_seq_folders(self):
        return self.__get_seq_folders(self.__path)

    def __get_seq_folders(self, root):
        result = []
        if not os.path.exists(root):
            logging.error("Watch directory {} doesn't exist".format(root))
            return result

        for f in os.listdir(root):
            path = os.path.join(root, f)
            if os.path.isdir(path):
                seq_folder = SeqFolderFactory.create(root, f)
                if seq_folder:
                    result.append(seq_folder)
                else:
                    result += self.__get_seq_folders(path)
        return result

    @staticmethod
    def __read_file(file_with_history):
        result = []
        if not os.path.isfile(file_with_history):
            return result
        with open(file_with_history, "r") as f:
            for line in f.readlines():
                result.append(line.strip())
        return result


class SeqFolderBase(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self, path):
        self._path = self.__get_abs_path(path).replace("\\", "/")

    def get_name(self):
        return os.path.basename(self._path)

    def get_path(self):
        return self._path

    @property
    @abstractmethod
    def is_valid(self):
        raise NotImplementedError()

    @property
    def is_uploading(self):
        return os.path.isfile(self._path + ".uploading")

    @property
    @abstractmethod
    def is_completed(self):
        raise NotImplementedError()

    def is_older_than(self, seconds):
        return time.time() - os.path.getmtime(self.__str__()) > seconds

    @abstractmethod
    def list_files(self):
        raise NotImplementedError()

    @staticmethod
    def __get_abs_path(path):
        abs_path = os.path.abspath(os.path.expanduser(path))
        if not os.path.isdir(abs_path):
            raise UploaderException("{} does not exist".format(abs_path))
        return abs_path


class WesSignateraSeqFolder(SeqFolderBase):
    FOLDER_REGEX = re.compile("[A-Z0-9]*_[0-9]{8}")

    def __init__(self, path):
        SeqFolderBase.__init__(self, path)
        self.__wes_subfolder = os.path.join(self._path, "WES_data")
        self.__qc_subfolder = os.path.join(self._path, "QC_reports")

    @property
    def is_valid(self):
        if WesSignateraSeqFolder.FOLDER_REGEX.match(self.get_name()) is None \
                or not os.path.isdir(self.__wes_subfolder) \
                or not os.path.isdir(self.__qc_subfolder):
            return False

        if len(self.__list_from_wes()) == 0:
            logging.info("{} does not contain any data".format(self.__wes_subfolder))
            return False
        if len(self.__list_from_qc_reports()) == 0:
            logging.info("{} does not contain any data".format(self.__qc_subfolder))
            return False

        success = True
        for f in self.list_files():
            if not f.is_valid:
                logging.info("{} has invalid format".format(f.get_full_path()))
                success = False
        return success

    def list_files(self):
        return self.__list_from_wes() + self.__list_from_qc_reports()

    def __list_from_wes(self):
        result = []
        for f in os.listdir(self.__wes_subfolder):
            result.append(WesDataFile(self.__wes_subfolder, f, self))
        return result

    def __list_from_qc_reports(self):
        result = []
        for f in os.listdir(self.__qc_subfolder):
            result.append(QcDataFile(self.__qc_subfolder, f, self))
        return result


class PersonalisSeqFolder(SeqFolderBase):
    FOLDER_REGEX = re.compile("[A-Z0-9]*_[0-9]{8}")

    def __init__(self, path):
        SeqFolderBase.__init__(self, path)
        self.__alignment_subfolder = os.path.join(self._path, '{}', "Alignment")
        self.__fastq_subfolder = os.path.join(self._path, '{}', "FASTQ")
        self.__qc_metrics_path = self._path

    @property
    def is_valid(self):
        if PersonalisSeqFolder.FOLDER_REGEX.match(self.get_name()) is None:
            return False

        if len(self.__list_qc_metrics(self._path)) == 0:
            logging.info("{} does not contain any data".format(self.__qc_metrics_path))
            return False

        for folder in os.listdir(self._path):
            if not os.path.isdir(os.path.join(self._path, folder)):
                continue
            if not os.path.isdir(self.__alignment_subfolder.format(folder)):
                return False
            if not os.path.isdir(self.__fastq_subfolder.format(folder)):
                return False
            if len(self.__list_from(self.__alignment_subfolder.format(folder))) == 0:
                logging.info("{} does not contain any data".format(self.__alignment_subfolder))
                return False
            if len(self.__list_from(self.__fastq_subfolder.format(folder))) == 0:
                logging.info("{} does not contain any data".format(self.__fastq_subfolder))
                return False

        success = True
        for f in self.list_files():
            if not f.is_valid:
                logging.info("{} has invalid format".format(f.get_full_path()))
                success = False
        return success

    def list_files(self):
        list_files = self.__list_qc_metrics(self._path)
        for folder in os.listdir(self._path):
            if not os.path.isdir(os.path.join(self._path, folder)):
                continue
            list_files.extend(self.__list_from(self.__alignment_subfolder.format(folder)))
            list_files.extend(self.__list_from(self.__fastq_subfolder.format(folder)))
        return list_files

    def __list_from(self, subfolder):
        result = []
        for f in os.listdir(subfolder):
            result.append(PersonalisDataFile(subfolder, f, self))
        return result

    def __list_qc_metrics(self, path):
        result = []
        for f in os.listdir(path):
            if os.path.isfile(os.path.join(path, f)):
                result.append(PersonalisQcDataFile(path, f, self))
        return result


class RawSeqFolder(SeqFolderBase):
    def __init__(self, path):
        super(RawSeqFolder, self).__init__(path)

    @property
    def is_valid(self):
        return True

    def list_files(self):
        result = []
        for root, subdirs, files in os.walk(self._path):
            for f in files:
                result.append(RawDataFile(root, f, self))
        return result


class BgiSeqFolder(SeqFolderBase):
    FOLDER_REGEX = re.compile("V[A-Za-z0-9]{9}")

    def __init__(self, path):
        super(BgiSeqFolder, self).__init__(path)
        self.__lane_folders = [os.path.join(self._path, lane) for lane in ["L01", "L02", "L03", "L04"]]
        self.__wes_subfolder = os.path.join(self._path, "WES_data")
        self.__qc_subfolder = os.path.join(self._path, "QC_reports")

    @property
    def is_completed(self):
        if self.is_valid:
            return all([os.path.isfile(os.path.join(lane_folder, "{}_{}_FileInfo.csv".format(os.path.basename(self._path), os.path.basename(lane_folder)))) for lane_folder in self.__lane_folders])
        return False

    @property
    def is_valid(self):
        if (BgiSeqFolder.FOLDER_REGEX.match(self.get_name())
                and all([os.path.join(self._path, folder) in self.__lane_folders for folder in os.listdir(self._path) if os.path.isdir(os.path.join(self._path, folder))])):
            return True
        return False

    def list_files(self):
        result = []
        for root, subdirs, files in os.walk(self._path):
            for f in files:
                result.append(RawDataFile(root, f, self))
        return result


class SeqFolderFactory(object):
    @staticmethod
    def create(base_path, name):
        if BgiSeqFolder.FOLDER_REGEX.match(name):
            return BgiSeqFolder(os.path.join(base_path, name))
        if WesSignateraSeqFolder.FOLDER_REGEX.match(name):
            return WesSignateraSeqFolder(os.path.join(base_path, name))
        return None

    @staticmethod
    def create_from_full_path(full_path):
        base_path, name = SeqFolderFactory.__parse_full_path(full_path)
        return SeqFolderFactory.create(base_path, name)

    @staticmethod
    def __parse_full_path(full_path):
        path = full_path.rstrip('/')
        return os.path.dirname(path), os.path.basename(path)



#!/usr/bin/env python

import os
from abc import ABCMeta, abstractmethod

import dxpy

from soyuz import __version__ as version
from soyuz.dx.variables import Type, Property


class SentinelBase(object):
    __metaclass__ = ABCMeta

    DATA_KEY = "data"
    METRICS_KEY = "run_metrics"
    DX_LINK_KEY = "$dnanexus_link"

    def __init__(self, basedir, name):
        self._dxrecord = dxpy.new_dxrecord(types=[Type.UPLOAD_SENTINEL],
                                           folder=os.path.join(basedir, name).replace("\\", "/"),
                                           name="{}_upload_sentinel".format(name),
                                           properties={Property.RUN_FOLDER: name,
                                                       Property.VERSION: version},
                                           parents=True)

    @abstractmethod
    def add_file(self, data_file, file_id):
        raise NotImplementedError()

    def get_id(self):
        return self._dxrecord.get_id()

    def close(self):
        self._dxrecord.close()


class WesSignateraSentinel(SentinelBase):
    def add_file(self, data_file, file_id):
        details = self._dxrecord.get_details()
        sample_id = data_file.get_sample_id()
        if sample_id:
            self._dxrecord.add_tags([sample_id])
            self.__add_data_file_details(details, sample_id, file_id)
        else:
            self.__add_metrics_details(details, file_id)
        self._dxrecord.set_details(details)

    @staticmethod
    def __add_data_file_details(details, sample_id, file_id):
        if SentinelBase.DATA_KEY not in details:
            details[SentinelBase.DATA_KEY] = {}
        data_details = details[SentinelBase.DATA_KEY]
        if sample_id not in data_details:
            data_details[sample_id] = []
        data_details[sample_id].append({SentinelBase.DX_LINK_KEY: file_id})

    @staticmethod
    def __add_metrics_details(details, file_id):
        if SentinelBase.METRICS_KEY not in details:
            details[SentinelBase.METRICS_KEY] = []
        details[SentinelBase.METRICS_KEY].append({SentinelBase.DX_LINK_KEY: file_id})


class PersonalisSentinel(SentinelBase):
    def add_file(self, data_file, file_id):
        details = self._dxrecord.get_details()
        sample_id = data_file.get_sample_id()
        if sample_id:
            self._dxrecord.add_tags([sample_id])
            self.__add_data_file_details(details, sample_id, file_id)
        else:
            self.__add_metrics_details(details, file_id)
        self._dxrecord.set_details(details)

    @staticmethod
    def __add_data_file_details(details, sample_id, file_id):
        if SentinelBase.DATA_KEY not in details:
            details[SentinelBase.DATA_KEY] = {}
        data_details = details[SentinelBase.DATA_KEY]
        if sample_id not in data_details:
            data_details[sample_id] = []
        data_details[sample_id].append({SentinelBase.DX_LINK_KEY: file_id})

    @staticmethod
    def __add_metrics_details(details, file_id):
        if SentinelBase.METRICS_KEY not in details:
            details[SentinelBase.METRICS_KEY] = []
        details[SentinelBase.METRICS_KEY].append({SentinelBase.DX_LINK_KEY: file_id})


class RawSentinel(SentinelBase):
    def add_file(self, data_file, file_id):
        pass



#!/usr/bin/env python

import logging
import os
import time
import subprocess
from abc import ABCMeta, abstractmethod

import dxpy

from soyuz.dx.sentinels import WesSignateraSentinel, RawSentinel, PersonalisSentinel
from soyuz.dx.variables import Type, Property
from soyuz.data.folders import *
from soyuz.utils import UploaderException


class WatchUploader(object):
    __metaclass__ = ABCMeta

    def __init__(self, dx, basedir, interval):
        self._dx = dx
        self._basedir = basedir
        self.interval = interval
        self.ua_path = None
        self.ua_parameters = None

    def watch(self, watch_dir):
        while True:
            for seq_folder in watch_dir.get_seq_folders():
                try:
                    uploader = DxUploaderFactory.create(self._dx, self._basedir, seq_folder)
                    uploader.ua_path = self.ua_path
                    uploader.ua_parameters = self.ua_parameters
                    uploader.upload(seq_folder)
                except UploaderException as e:
                    logging.error("{}. Skipping".format(e))
                    continue
            time.sleep(self.interval)


class DxUploaderBase(object):
    __metaclass__ = ABCMeta

    def __init__(self, dx, basedir):
        self._dx = dx
        self._basedir = basedir
        self.ua_path = None
        self.ua_parameters = None

    def upload(self, seq_folder):
        self.__validate_target_dir(seq_folder)

        logging.info("Starting upload for {}".format(seq_folder.get_name()))
        sentinel = self._create_sentinel(seq_folder.get_name())
        for data_file in seq_folder.list_files():
            remote_folder = os.path.join(self._basedir,
                                         data_file.get_seq_folder_name(),
                                         data_file.get_relative_path()).replace("\\", "/")
            types = self._get_additional_types(data_file)
            types.append(Type.UPLOAD_DATA)
            properties = self._get_additional_properties(data_file, seq_folder.get_name())
            properties[Property.RUN_FOLDER] = seq_folder.get_name()
            if self.ua_path and os.path.isfile(self.ua_path):
                dx_file = self.ua_upload_file_with_details(data_file, remote_folder, types, properties)
            else:
                dx_file = self.dx_upload_file_with_details(data_file, remote_folder, types, properties)
            sentinel.add_file(data_file, dx_file.get_id())
        sentinel.close()
        logging.info("{} has been successfully uploaded".format(seq_folder.get_name()))

    def __validate_target_dir(self, folder):
        if not folder.is_valid:
            raise UploaderException(
                    "{} is not valid".format(folder.get_name()))
        project = dxpy.DXProject(self._dx.get_project_id())
        try:
            entities = project.list_folder(os.path.join(self._basedir, folder.get_name()))
            if len(entities["objects"]) > 0 or len(entities["folders"]) > 0:
                raise UploaderException(
                    "{} already exists under {}".format(folder.get_name(), self._basedir))
        except dxpy.exceptions.ResourceNotFound:
            pass

    def ua_upload_file_with_details(self, data_file, remote_folder, types, properties):
        args = [r'"{}"'.format(self.ua_path), data_file.get_full_path().replace("(", "\(").replace(")", "\)")]
        args.extend(["--auth-token", self._dx.get_token()])
        args.extend(["-p", self._dx.get_project_id()])
        args.extend(["-f", remote_folder])
        args.exists([self.ua_parameters])
        args.extend(["--type {}".format(_type) for _type in types])
        args.extend(["--property {}={}".format(key, val) for key, val in properties.items()])
        subprocess.check_output(" ".join(args), shell=True)
        objects = list(dxpy.bindings.find_data_objects(classname="file",
                                                       folder=remote_folder,
                                                       describe=True,
                                                       project=self._dx.get_project_id(),
                                                       return_handler=True))
        return objects[0] if objects else None

    def dx_upload_file_with_details(self, data_file, remote_folder, types, properties):
        dx_file = self.upload_file(data_file, remote_folder)

        if dx_file:
            dx_file.add_types(types)
            dx_file.set_properties(properties)
            dx_file.close()
        else:
            raise UploaderException("Failed to upload {}".format(data_file.get_full_path()))

        return dx_file

    def upload_file(self, data_file, remote_folder):
        logging.info("Uploading {} to {}".format(data_file.get_full_path(), remote_folder))
        return dxpy.upload_local_file(data_file.get_full_path(),
                                      folder=remote_folder,
                                      keep_open=True,
                                      parents=True)

    @abstractmethod
    def _create_sentinel(self, seq_folder_name):
        raise NotImplementedError()

    @abstractmethod
    def _get_additional_types(self, data_file):
        raise NotImplementedError()

    @abstractmethod
    def _get_additional_properties(self, data_file, seq_folder_name):
        raise NotImplementedError()


class WesSignateraDxUploader(DxUploaderBase):
    SEQ_FOLDER_TYPE = WesSignateraSeqFolder

    def _create_sentinel(self, seq_folder_name):
        return WesSignateraSentinel(self._basedir, seq_folder_name)

    def _get_additional_types(self, data_file):
        types = []
        data_type = data_file.get_type()
        if data_type:
            types.append(data_type)
            if data_type == Type.CSV and data_file.get_name().startswith("WES-QCMetrics"):
                types.append(Type.WESQCREPORT)
        return types

    def _get_additional_properties(self, data_file, seq_folder_name):
        properties = {}
        if data_file.get_sample_id():
            properties[Property.SAMPLE_REFERENCE] = "{}/{}".format(seq_folder_name, data_file.get_sample_id())
        return properties


class PersonalisDxUploader(DxUploaderBase):
    SEQ_FOLDER_TYPE = PersonalisSeqFolder

    def _create_sentinel(self, seq_folder_name):
        return PersonalisSentinel(self._basedir, seq_folder_name)

    def _get_additional_types(self, data_file):
        types = []
        data_type = data_file.get_type()
        if data_type:
            types.append(data_type)
            if data_type == Type.CSV and data_file.get_name().startswith("QCMetrics"):
                types.append(Type.QCREPORT)
        return types

    def _get_additional_properties(self, data_file, seq_folder_name):
        properties = {}
        if data_file.get_sample_id():
            properties[Property.SAMPLE_REFERENCE] = "{}/{}".format(seq_folder_name, data_file.get_sample_id())
        return properties


class RawDxUploader(DxUploaderBase):
    SEQ_FOLDER_TYPE = RawSeqFolder

    def _create_sentinel(self, seq_folder_name):
        return RawSentinel(self._basedir, seq_folder_name)

    def _get_additional_types(self, data_file):
        return []

    def _get_additional_properties(self, data_file, seq_folder_name):
        return {}


class BgiDxUploader(DxUploaderBase):
    SEQ_FOLDER_TYPE = BgiSeqFolder

    def _create_sentinel(self, seq_folder_name):
        return RawSentinel(self._basedir, seq_folder_name)

    def _get_additional_types(self, data_file):
        return []

    def _get_additional_properties(self, data_file, seq_folder_name):
        return {}


class DxUploaderFactory(object):
    @staticmethod
    def create(dx, basedir, seq_folder):
        if isinstance(seq_folder, BgiDxUploader.SEQ_FOLDER_TYPE):
            return BgiDxUploader(dx, basedir)
        if isinstance(seq_folder, RawDxUploader.SEQ_FOLDER_TYPE):
            return RawDxUploader(dx, basedir)
        if isinstance(seq_folder, WesSignateraDxUploader.SEQ_FOLDER_TYPE):
            return WesSignateraDxUploader(dx, basedir)
        raise UploaderException(
                "Uploader for the folder {} was not found".format(seq_folder.get_name()))



#!/usr/bin/env python


class Property(object):
    JOB_ID = "jobId"
    RUN_FOLDER = "runFolder"
    VERSION = "version"
    PRODUCT = "product"
    SAMPLE_REFERENCE = "uploadSampleReference"


class Type(object):
    UPLOAD_DATA = "UploadData"
    UPLOAD_SENTINEL = "UploadSentinel"
    UPLOAD_JOB = "UPLOAD"
    BAM = "bam"
    FASTQ = "fastq"
    CSV = "csv"
    PDF = "pdf"
    XLSX = "xlsx"
    WESQCREPORT = "WESQcReport"
    QCREPORT = "QcReport"


class State(object):
    RUNNING = "running"
    WAITING = "waiting_on_input"
    TERMINATED = "terminated"
    DONE = "done"
    OPEN = "open"
    CLOSING = "closing"
    CLOSED = "closed"



#!/usr/bin/env python

import dxpy

from soyuz.utils import UploaderException


class DxContext(object):
    def __init__(self, token):
        self.__token = token
        dxpy.set_security_context({'auth_token_type': 'Bearer', 'auth_token': token})
        projects = self.__get_projects()
        size = len(projects)
        if size == 0 or size > 1:
            raise UploaderException("Auth Token must have access to exactly 1 project with UPLOAD permission.")
        self.__project = projects[0]
        dxpy.set_project_context(self.__project)
        dxpy.set_workspace_id(self.__project)

    def get_token(self):
        return self.__token

    def get_project_id(self):
        return self.__project

    def get_project(self):
        return dxpy.DXProject(self.get_project_id())

    @staticmethod
    def __get_projects():
        result = []
        try:
            for project in dxpy.bindings.search.find_projects(level='UPLOAD'):
                result.append(str(project['id']))
        except dxpy.exceptions.InvalidAuthentication:
            pass
        return result



#!/usr/bin/env python

import logging

from soyuz.configuration import Parameters
from soyuz.configuration import Settings
from soyuz.data.daemons import Daemon
from soyuz.data.folders import *
from soyuz.dx.context import DxContext
from soyuz.dx.uploaders import *
from soyuz.utils import UploaderException


def main():
    try:
        params = Parameters()
        settings = Settings()

        settings.logging.initialize()

        if params.get_action() in [Parameters.UPLOAD_ACTION, Parameters.WATCH_ACTION]:

            token = params.get_token()
            if not token:
                token = settings.get_token()
            if not token:
                raise UploaderException("Token was not specified")

            dx_context = DxContext(token)

            folder = None
            uploader = None

            if params.get_action() == Parameters.UPLOAD_ACTION:
                if params.get_upload_type() == Parameters.WES_SIGNATERA_UPLOAD:
                    uploader = WesSignateraDxUploader(dx_context, settings.get_base_dir())
                    folder = WesSignateraSeqFolder(params.get_folder())
                elif params.get_upload_type() == Parameters.BGI_UPLOAD:
                    uploader = BgiDxUploader(dx_context, settings.get_base_dir())
                    folder = BgiSeqFolder(params.get_folder())
                elif params.get_upload_type() == Parameters.RAW_UPLOAD:
                    uploader = RawDxUploader(dx_context, settings.get_base_dir())
                    folder = RawSeqFolder(params.get_folder())
                if params.get_upload_type() == Parameters.PERSONALIS_UPLOAD:
                    uploader = PersonalisDxUploader(dx_context, settings.get_base_dir())
                    folder = PersonalisSeqFolder(params.get_folder())

                if not folder or not uploader:
                    raise UploaderException("Incorrect upload type")

                if not folder.is_valid:
                    raise UploaderException("Data folder is not in a valid state")

                uploader.ua_path = settings.get_ua_path()
                uploader.ua_parameters = settings.get_ua_parameters()
                uploader.upload(folder)

            elif params.get_action() == Parameters.WATCH_ACTION:

                uploader = WatchUploader(dx_context, settings.get_base_dir(), params.get_interval())
                uploader.ua_path = settings.get_ua_path()
                uploader.ua_parameters = settings.get_ua_parameters()

                if params.get_watch_type() == Parameters.START_WATCH:
                    if params.foreground():
                        uploader.watch(WatchDirectory(params.get_folder()))
                    else:
                        Daemon(uploader.watch, WatchDirectory(params.get_folder())).start()
                elif params.get_watch_type() == Parameters.STOP_WATCH:
                    Daemon().stop()
                elif params.get_watch_type() == Parameters.STATUS_WATCH:
                    Daemon().status()

        elif params.get_action() == Parameters.CONFIG_ACTION:

            if params.get_config_action() == Parameters.GET_CONFIG:
                if params.get_config_parameter_key() == "token":
                    logging.info(settings.get_token())
                elif params.get_config_parameter_key() == "basedir":
                    logging.info(settings.get_base_dir())
                elif params.get_config_parameter_key() == "interval":
                    logging.info(settings.get_interval())
                elif params.get_config_parameter_key() == "ua_path":
                    logging.info(settings.get_ua_path())
                elif params.get_config_parameter_key() == "ua_parameters":
                    logging.info(settings.get_ua_parameters())

            elif params.get_config_action() == Parameters.SET_CONFIG:
                if params.get_config_parameter_key() == "token":
                    settings.set_token(params.get_config_parameter_value())
                elif params.get_config_parameter_key() == "basedir":
                    settings.set_basedir(params.get_config_parameter_value())
                elif params.get_config_parameter_key() == "interval":
                    settings.set_interval(params.get_config_parameter_value())
                elif params.get_config_parameter_key() == "ua_path":
                    settings.set_ua_path(params.get_config_parameter_value())
                elif params.get_config_parameter_key() == "ua_parameters":
                    settings.set_ua_parameters(params.get_config_parameter_value())
                else:
                    raise UploaderException("There is no parameter '{}'".format(params.get_config_parameter_key()))
                settings.dump()
                logging.info("Set {} to {}".format(params.get_config_parameter_key(), params.get_config_parameter_value()))

    except UploaderException as e:
        logging.error(e)
        quit(1)


if __name__ == "__main__":
    main()
